var async = require('async');
var del = require('del');

var defaultDir = '/usr/src/app/userdata/networks';

function checkBoundary(client, canvasId ,elementId, direction) {
    var boundary;
    client.getElementSize(canvasId, function(result) {
        client.getElementSize(elementId, function(elementSize) {
            if(direction === 'right') {
                boundary = parseInt((result.value.width - elementSize.value.width) / 10, 10) * 10;
                client.expect.element(elementId).to.have.attribute('style').contains('left: '+ boundary +'px;');
            } else if(direction === 'down') {
                boundary = parseInt((result.value.height - elementSize.value.height) / 10, 10) * 10;
                client.expect.element(elementId).to.have.attribute('style').contains('top: '+ boundary +'px;');
            }

        });
    });
}

module.exports = {
    '@tags': ['cl_canvassDrag'],

    before: function (client) {
        client
            .url('http://localhost:3000')
            .waitForElementVisible('#topmenu', 1000);
    },

    after: function (client) {
        // Clean up
        del.sync(defaultDir + '/*');

        client.end();
    },

    /**
     * Test: 1-001
     * Expects:
     *  - Hark Designer should display all available node groups according to their def files.
     */
    "1-001" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // 1-001
                for (var i = 1; i <= 33; i++) {
                    canvassLayout.expect.element('#nodelist > li:nth-child(' + i + ')').to.be.present;
                }
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-002
     * Expects:
     *  - Node "LocalizeMusic" will be added into the canvass
     *  - Node name must be "node_LocalizeMusic_1"
     */
    "1-002" : function (client) {
        var canvassLayout = client.page.canvassLayout();
        async.series([
            function (callback) {
                canvassLayout.clearValue('@inputSearch');
                canvassLayout.setValue('@inputSearch', 'localize');
                callback();
            },
            function (callback) {
                canvassLayout.clickLocalizeMusic(function () {
                    callback();
                });
            },
            function (callback) {
                // 1-002
                canvassLayout.expect.element('@localizeMusicNode').to.be.present;
                canvassLayout.expect.element('@localizeMusicNodeId').text.to.equal('node_LocalizeMUSIC_1');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-003
     * Expects:
     *  - Node must not be dragged outside the bounds of the canvass
     */
    "1-003" : function (client) {
        var canvassLayout = client.page.canvassLayout();
        async.series([
            // Move node element to the leftmost part of the canvass
            function (callback) {
                client
                    .moveToElement('#node_LocalizeMUSIC_1', 170, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 250, 150)
                    .mouseButtonUp('left', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-003
                canvassLayout.expect.element('@localizeMusicNode').to.have.attribute('style').contains('left: 0px;');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-004
     * Expects:
     *  - Node must not be dragged outside the bounds of the canvass
     */
    "1-004" : function (client) {
        var canvassLayout = client.page.canvassLayout();
        async.series([
            // Move node element to the topmost part of the canvass
            function (callback) {
                client
                    .moveToElement('#node_LocalizeMUSIC_1', 0, 70)
                    .mouseButtonDown('left')
                    .moveToElement('body', 250, 150)
                    .mouseButtonUp('left', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-004
                canvassLayout.expect.element('@localizeMusicNode').to.have.attribute('style').contains('top: 0px;');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-005
     * Expects:
     *  - Node must not be dragged outside the bounds of the canvass
     */
    "1-005" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            // Move node element to the rightmost part of the canvass
            function (callback) {
                client
                    .moveToElement('#node_LocalizeMUSIC_1', 0, 70)
                    .mouseButtonDown('left')
                    .moveToElement('body', 6500, 150)
                    .mouseButtonUp('left')
                    .getLocationInView('#node_LocalizeMUSIC_1',  function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-005
                checkBoundary(client, '#canvas2_MAIN', '#node_LocalizeMUSIC_1', 'right');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-006
     * Expects:
     *  - Node must not be dragged outside the bounds of the canvass
     */
    "1-006" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            // Move node element to the bottommost part of the canvass
            function (callback) {
                client
                    .moveToElement('#node_LocalizeMUSIC_1', 0, 70)
                    .mouseButtonDown('left')
                    .moveToElement('body', 250, 6500)
                    .mouseButtonUp('left')
                    .getLocationInView('#node_LocalizeMUSIC_1',  function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-006
                checkBoundary(client, '#canvas2_MAIN', '#node_LocalizeMUSIC_1', 'down');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-007
     * Expects:
     *  - Another "LocalizeMusic" node will be added into the canvass
     *  - Name of newly added node must be "node_LocalizeMusic_2"
     */
    "1-007" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            // Move node element to the bottommost part of the canvass
            function (callback) {
                // Move node element to the leftmost part of the canvass
                client
                    .moveToElement('#node_LocalizeMUSIC_1', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', -5200, -5170)
                    .mouseButtonUp('left')
                    .getLocationInView('#node_LocalizeMUSIC_1', function () {
                        callback();
                    });
            },

            function (callback) {
                canvassLayout.clickLocalizeMusic(function () {
                    callback();
                });
            },

            function (callback) {
                // 1-007
                canvassLayout.expect.element('@localizeMusicNode_2').to.be.present;
                canvassLayout.expect.element('@localizeMusicNodeId_2').text.to.equal('node_LocalizeMUSIC_2');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-008
     * Expects:
     *  - "node_LocalizeMusic_2" should be positioned at the bottom right of "node_LocalizeMusic_1"
     */
    "1-008" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                client.click('#node_LocalizeMUSIC_2');
                callback();
            },
            function (callback) {
                for(var i = 0; i < 2; i++) {
                    client.keys([client.Keys.DOWN_ARROW]);
                }
                callback();
            },
            function (callback) {
                client.getLocation("#node_LocalizeMUSIC_1", function (result1) {
                    client.getLocation("#node_LocalizeMUSIC_2", function (result2) {
                        this.assert.equal((result2.value.y - 100), result1.value.y);
                    });
                });

                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-009
     * Expects:
     *  - Both nodes will appear with shadows which means they are selected.
     */
    "1-009" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                client
                    .moveToElement('#sheet_MAIN', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 350, 350)
                    .mouseButtonUp('left', function () {
                        callback();
                    });
            },

            function (callback) {
                // 1-009
                client.expect.element('#node_LocalizeMUSIC_1').to.have.attribute('class').contains('nodeSelect');
                client.expect.element('#node_LocalizeMUSIC_2').to.have.attribute('class').contains('nodeSelect');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-010
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     *
     * Remarks: Issue needs to be fixed.
     */
    "1-010" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            // TO BE FIX (left)
            /*function (callback) {
             // Move to the leftmost part of the canvass by dragging node_LocalizeMUSIC_2
             client
             .moveToElement('#node_LocalizeMUSIC_2', 0, 0)
             .mouseButtonDown('left')
             .moveToElement('body', 0, 250)
             .mouseButtonUp('left')
             .getLocationInView('#node_LocalizeMUSIC_2', function () {
             callback();
             });
             },
             // To be fix 1-010
             function (callback) {
             client.expect.element('#node_LocalizeMUSIC_1').to.have.attribute('style').contains('left: 5.48438px;');
             callback();
             },*/
        ], function (err, results) {

        })
    },


    /**
     * Test: 1-011
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     */
    "1-011" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Move to the leftmost part of the canvass by dragging node_LocalizeMUSIC_1
                client
                    .moveToElement('#node_LocalizeMUSIC_1', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', -6500, 250)
                    .mouseButtonUp('left')
                    .getLocationInView('#node_LocalizeMUSIC_1', function () {
                        callback();
                    });
            },

            function (callback) {
                // 1-011
                client.expect.element('#node_LocalizeMUSIC_1').to.have.attribute('style').contains('left: 0px;');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-012
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     *
     * Remarks: Issue needs to be fixed.
     */
    "1-012" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            //  TO BE FIX (top)
            /*function (callback) {
             // Move nodes to the topmost of the canvass dragging the node_LocalizeMUSIC_2
             client
             .moveToElement('#node_LocalizeMUSIC_2', 0, 0)
             .mouseButtonDown('left')
             .moveToElement('body', 250, 0)
             .mouseButtonUp('left')
             .getLocationInView('#node_LocalizeMUSIC_2', function () {
             callback();
             });
             },

             function (callback) {
             // 1-012 ---> To be fix
             // Expect node_LocalizeMUSIC_1 should not get outside the boundary
             client.expect.element('#node_LocalizeMUSIC_1').to.have.attribute('style').contains('top: 2.96875px;');
             callback();
             },*/
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-013
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     */
    "1-013" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Select nodes by dragging mouse in the canvass
                client
                    .moveToElement('#sheet_MAIN', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 350, 350)
                    .mouseButtonUp('left', function () {
                        callback();
                    });
            },

            function (callback) {
                // Move nodes to the topmost of the canvass dragging the node_LocalizeMUSIC_1
                client
                    .moveToElement('#node_LocalizeMUSIC_1', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 0, -600)
                    .mouseButtonUp('left')
                    .getLocationInView('#node_LocalizeMUSIC_1', function () {
                        callback();
                    });
            },

            function (callback) {
                // 1-013
                // Expect node_LocalizeMUSIC_1 should not get outside the boundary
                client.expect.element('#node_LocalizeMUSIC_1').to.have.attribute('style').contains('top: 0px;');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-014
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     */
    "1-014" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Move nodes to the rightmost part of the canvass dragging the node_LocalizeMUSIC_2
                client
                    .moveToElement('#node_LocalizeMUSIC_2', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 6500, 250)
                    .mouseButtonUp('left')
                    .getLocationInView('#node_LocalizeMUSIC_2', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-014
                // Expect node_LocalizeMUSIC_2 should not get outside the boundary
                checkBoundary(client, '#canvas_MAIN', '#node_LocalizeMUSIC_2', 'right');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-015
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     *
     * Remarks: Issue needs to be fixed.
     */
    "1-015" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            // TO BE FIX (right)
            /*function (callback) {
             // Move nodes to the rightmost part of the canvass dragging the node_LocalizeMUSIC_1
             client
             .moveToElement('#node_LocalizeMUSIC_1', 0, 0)
             .mouseButtonDown('left')
             .moveToElement('body', 1500, 250)
             .mouseButtonUp('left')
             .getLocationInView('#node_LocalizeMUSIC_1', function () {
             callback();
             });
             },
             function (callback) {
             // 1-015
             // Expect node_LocalizeMUSIC_1 should not get outside the boundary
             client.expect.element('#node_LocalizeMUSIC_1').to.have.attribute('style').contains('left: 5810.48px;');
             callback();
             },*/
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-016
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     *
     * Remarks: Issue needs to be fixed.
     */
    "1-016" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Move nodes to the bottomost part of the canvass dragging the node_LocalizeMUSIC_2
                client
                    .moveToElement('#node_LocalizeMUSIC_2', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 1500, 6500)
                    .mouseButtonUp('left')
                    .getLocationInView('#node_LocalizeMUSIC_2', function () {
                        callback();
                    });
            },

            function (callback) {
                // 1-016
                // Expect node_LocalizeMUSIC_2 should not get outside the boundary
                checkBoundary(client, '#canvas2_MAIN', '#node_LocalizeMUSIC_2', 'down');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-017
     * Expects:
     *  - Selected nodes must not be dragged outside the bounds of the canvass
     *
     * Remarks: Issue needs to be fixed.
     */
    "1-017" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            //TO BE FIX (bottom)
            /*function (callback) {
             // Move nodes to the bottommost part of the canvass dragging the node_LocalizeMUSIC_1
             client
             .moveToElement('#node_LocalizeMUSIC_1', 0, 0)
             .mouseButtonDown('left')
             .moveToElement('body', 1500, 6100)
             .mouseButtonUp('left')
             .getLocationInView('#node_LocalizeMUSIC_1', function () {
             callback();
             });
             },

             function (callback) {
             // 1-017
             // Expect node_LocalizeMUSIC_2 should not get outside the boundary
             client.expect.element('#node_LocalizeMUSIC_2').to.have.attribute('style').contains('top: 5913.97px;');
             callback();
             },*/
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-018
     * Expects:
     *  - "Sample" sheet will be added with blank canvass.
     */
    "1-018" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                canvassLayout.clickAddSheet(function () {
                    callback();
                })
            },
            function (callback) {
                canvassLayout.expect.element('@addSheetDialog').to.be.visible;
                canvassLayout.setValue('@addSheetInput', 'Sample', function () {
                    callback();
                });
            },
            function (callback) {
                canvassLayout.clickAddSheetBtn(function () {
                    callback();
                })
            },
            function (callback) {
                // 1-018
                canvassLayout.expect.element('@sheetTab').to.be.visible;
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-019
     * Expects:
     *  - In the "Sample" sheet, "node_LocalizeMusic_3" should be added.
     */
    "1-019" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                canvassLayout.clickLocalizeMusic(function () {
                    callback();
                })
            },
            function (callback) {
                // 1-019
                canvassLayout.expect.element('@localizeMusicNode_3').to.be.visible;
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-020
     * Expects:
     *  - In the "Sample" sheet, "node_LocalizeMusic_4" should be added.
     */
    "1-020" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                canvassLayout.clickLocalizeMusic(function () {
                    callback();
                })
            },
            function (callback) {
                // 1-020
                canvassLayout.expect.element('@localizeMusicNode_4').to.be.visible;
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-021
     * Expects:
     *  - A create note dialog will appear
     */
    "1-021" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                client.moveToElement('#canvas2_Sample', 50, 50)
                    .mouseButtonClick('right', function () {
                        callback();
                    })
            },
            function (callback) {
                canvassLayout.clickAddNote(function () {
                    callback();
                })
            },
            function (callback) {
                // 1-021
                canvassLayout.expect.element('@addNoteDialog').to.be.visible;
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-022
     * Expects:
     *  - Note displaying "Sample text" will be added into the canvass
     */
    "1-022" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                canvassLayout.setValue('@addNoteTextArea', 'Sample text', function () {
                    callback();
                });
            },
            function (callback) {
                // 1-022
                canvassLayout.expect.element('@addNoteOk').to.be.present;
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-023
     * Expects:
     *  - Selected note must not be dragged outside the bounds of the canvass
     */
    "1-023" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                canvassLayout.click('@addNoteOk', function() {
                    callback();
                });
            },

            function (callback) {
                // Move note to the leftmost part of the canvass
                client
                    .moveToElement('#note0', 170, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 250, 250)
                    .mouseButtonUp('left', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-023
                canvassLayout.expect.element('#note0').to.have.attribute('style').contains('left: 0px;');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-024
     * Expects:
     *  - Selected note must not be dragged outside the bounds of the canvass
     */
    "1-024" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Move note to the topmost part of the canvass
                client
                    .moveToElement('#note0', 170, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 250, 40)
                    .mouseButtonUp('left', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-024
                canvassLayout.expect.element('#note0').to.have.attribute('style').contains('top: 0px;');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-025
     * Expects:
     *  - Selected note must not be dragged outside the bounds of the canvass
     */
    "1-025" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Move note to the rightmost part of the canvass
                client
                    .moveToElement('#note0', 170, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 6200, 40)
                    .mouseButtonUp('left')
                    .getLocationInView('#note0', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-025
                checkBoundary(client, '#canvas2_Sample', '#note0', 'right');
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-026
     * Expects:
     *  - Selected note must not be dragged outside the bounds of the canvass
     */
    "1-026" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Move note to the bottommost part of the canvass
                client
                    .moveToElement('#note0', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 6200, 6550)
                    .mouseButtonUp('left')
                    .getLocationInView('#note0', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-026
                var boundary;
                client.getElementSize('#canvas2_Sample', function(result) {
                    client.getElementSize('#note0', function(elementSize) {
                        boundary = parseInt((result.value.height - elementSize.value.height) / 10, 10) * 10;
                        boundary+= 10;
                        client.expect.element('#note0').to.have.attribute('style').contains('top: '+ boundary +'px;');

                    });
                });
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-027
     * Expects:
     *  - Another note will added in the canvass with "Sample text2" as text
     */
    "1-027" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                client.moveToElement('#canvas_Sample', 350, 200)
                    .mouseButtonClick('right', function () {
                        callback();
                    })
            },
            function (callback) {
                canvassLayout.clickAddNote(function () {
                    callback();
                })
            },
            function (callback) {
                canvassLayout.setValue('@addNoteTextArea_1', 'Sample text2', function () {
                    canvassLayout.click('@addNoteOk_1', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                // 1-027
                canvassLayout.expect.element('#note1').to.be.visible;
                callback();
            }
        ], function (err, results) {

        })
    },

    /**
     * Test: 1-028
     * Expects:
     *  - The two notes should be selected.
     */
    "1-028" : function (client) {
        var canvassLayout = client.page.canvassLayout();

        async.series([
            function (callback) {
                // Move note to the bottommost part of the canvass
                client
                    .moveToElement('#note0', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', -4700, -5100)
                    .mouseButtonUp('left')
                    .getLocationInView('#note0', function () {
                        callback();
                    });
            },
            function (callback) {
                client.moveToElement('#canvas_Sample', 550, 550)
                    .mouseButtonClick('left', function () {
                        callback();
                    })
            },
            function (callback) {
                client
                    .moveToElement('#canvas2_Sample', 0, 0)
                    .mouseButtonDown('left')
                    .moveToElement('body', 550, 550)
                    .mouseButtonUp('left', function () {
                        callback();
                    });
            },
            function (callback) {
                // 1-028
                canvassLayout.expect.element('#note0').to.have.attribute('class').contains('noteSelected');
                canvassLayout.expect.element('#note1').to.have.attribute('class').contains('noteSelected');
                callback();
            }
        ], function (err, results) {

        })

        client.end();
    }
};