var async = require('async');
var moment = require('moment');
var fs = require('fs-extra');
var _ = require('underscore');
var del = require('del');

var defaultDir = '/usr/src/app/userdata/networks';
var dLoadDir = '/home/seluser/Downloads';
var fixtureDir = '/usr/src/app/automated-tests/lib/fixtures';

var fileListCount = 5;
var sampleFiles = [];

function screenshot (client) {
    var fileName = moment().format('YYYYMMDDHHmmss');
    var screenShotPath = './automated-tests/screenshots/';
    var fileFormat = '.png';
    var filePath = screenShotPath+fileName+fileFormat;

    client.saveScreenshot(filePath);
}

function getLatestFile (path) {
    var files = fs.readdirSync(path);

    files.sort(function(a, b) {
        return fs.statSync(path + a).mtime.getTime() -
            fs.statSync(path + b).mtime.getTime();
    });

    return files.reverse()[0];
}

function generateFileName (count) {
    var arr = [];
    var makeid = function () {
        var text = "";
        var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

        for( var i=0; i < 5; i++ )
            text += possible.charAt(Math.floor(Math.random() * possible.length));

        return text;
    };

    for (var i = 0; i < count; i++) {
        arr.push(makeid());
    }

    return arr;
}

function getPagedData (page) {
    var limit = 5;
    var totalPages = sampleFiles.length;
    var startIndex = (page - 1) * limit;
    var arr = [];
    var count = 0;

    for (var i = startIndex; i < totalPages; i++) {
        if (count < limit) {
            arr.push(sampleFiles[i]);
        }
        count++;
    }

    return arr;
}

function sortByType () {
    var temp = [];
    var temp2 = [];

    for (var i in sampleFiles) {
        var ext = sampleFiles[i].split('.').pop();

        switch(ext) {
            case 'n':
                    temp.push({name: sampleFiles[i], type: 'network'});
                break;
            default:
                    temp.push({name: sampleFiles[i], type: 'others'});
                break;
        }
    }

    var grpData = _.groupBy(_.sortBy(temp, "name"), "type");

    grpData = _.sortBy(grpData, function (data, index) {
        return index;
    });

    for (var j in grpData) {
        var temp = _.chain(grpData[j])
                        .map(function (data) {
                            return data.name;
                        })
                        .value();
        temp2 = temp2.concat(temp);
    }

    sampleFiles = temp2;
}

module.exports = {
    '@tags': ['fm_fileList'],

    before: function (client) {
        client
            .url('http://localhost:3000')
            .waitForElementVisible('#topmenu', 1000);
    },

    after: function (client) {
        // Clean up
        del.sync(defaultDir + '/*');
        del.sync(dLoadDir + '/*', {force: true});

        client.end();
    },

    /**
     *  Test: 1-001
     *  Expects:
     *      - File Manager dialog will appear
     *      - In Path field, the Hark Designer's userdata/networks directory will be displayed
     *      - In the file list, all files in the userdata/networks directory should be displayed and should be paginated properly
     *      - If there are no files in the userdata/networks directory, "No Records Found" label will be visible in the file list
     */
    "1-001": function (client) {
        var fileManagement = client.page.fileManagement();

        // Open file manager
        fileManagement.open();

        client.pause(1000, function () {
            // Assert file manager to appear
            fileManagement.expect.element('@fileManagerDialog').to.be.visible;

            // Assert file path to have the default directory
            fileManagement.expect.element('@dataPath').to.have.value.which.contains(defaultDir);

            // Assert file displays in file list
            // Assumes file list is blank
            fileManagement.expect.element('@fileList').text.to.contain('No Records Found');

            // Assert files are paginated properly
            // Assumes there is no file, page should only contain one(1)
            fileManagement.expect.element('@pages').text.not.to.contain('2');
        });
    },

    /**
     *  Test: 1-002
     *  Expects:
     *      - In the file list, files that contains "a" will be listed in alphabetical order according to type
     *      - If there are no files matching the search key, "No Records Found" label will be visible in the file list
     */
    "1-002": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Clear value for filter
                fileManagement.clearValue('@filter');
                // Select filter
                fileManagement.moveToElement('@filter', 10, 10);

                callback();
            },
            function (callback) {
                // Enter 'a' key
                client.keys('a',function () {
                    this.pause(1000);
                    callback();
                });
            },
            function (callback) {
                // Assert file list if file with 'a' is displayed
                // Assumes result to be blank
                fileManagement.expect.element('@fileList').text.to.contain('No Records Found');

                callback();
            },
            function (callback) {
                // Clear value for filter
                fileManagement.clearValue('@filter');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-003
     *  Expects:
     *      - In Hark Designer, no change should be observed
     */
    "1-003": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = defaultDir + '/sample.txt';

        fs.outputFileSync(path, "");

        // Assert file list if there's nothing change
        // Assumes result to be blank
        fileManagement.expect.element('@fileList').text.to.contain('No Records Found');
    },

    /**
     *  Test 1-004
     *  Expects
     *      - "sample.txt" should already be displayed in the file list in alphabetical order
     *      - Its type should be "unknown"
     *      - Action button should only be delete and download
     */
    "1-004": function (client) {
        var fileManagement = client.page.fileManagement();

        // Clicks reload button
        fileManagement.clickReload();

        client.pause(1000);

        // Assert file list that changes are applied
        // Assumes that file list is not empty
        fileManagement.expect.element('@fileList').text.to.contain('sample.txt');

        // Asserts file type to be unknown
        fileManagement.expect.element('@fileList').text.to.contain('others');

        // Asserts actions buttons to have only 'Delete' and 'Download'
        fileManagement.expect.element('@fileList').text.to.contain('Delete');
        fileManagement.expect.element('@fileList').text.to.contain('Download');
    },

    /**
     *  Test: 1-005
     *  Expects:
     *      - "sample.txt" will be downloaded by the browser
     */
    "1-005": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Click download button
                fileManagement.click('@dloadBtn', function () {
                    callback();
                });
            },
            function (callback) {
                // Checks the download directory if downloaded
                client.pause(1000, function () {
                    var file = getLatestFile(dLoadDir + '/');

                    // Asserts if downloaded file is in the filelist
                    fileManagement.expect.element('@fileList').text.to.contain(file).after(1000);

                    callback();
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-006
     *  Expects:
     *      - Browser's default dialog box will appear asking "Are you sure to delete file sample.txt?"
     *      - Action buttons in the dialog box are "OK" and "Cancel"
     */
    "1-006": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Clicks delete button
                fileManagement.click('@delBtn');

                callback();
            },
            function (callback) { // Test: 1-006
                // Asserts alert box appears
                client.getAlertText(function (result) {
                    // Assumes alert box appears if content is present
                    if (result.value === 'Are you sure to delete file sample.txt?') {
                        callback();
                    }
                    else { // Terminate test
                        client.assert.fail(result.value, "Are you sure to delete file sample.txt?", "Alert should be visible");
                    }
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-007
     *  Expects:
     *      - Dialog box will disappear.
     */
    "1-007": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-007
                client.pause(1000);

                // Dismiss alert box
                client.dismissAlert(function () {
                    callback();
                });
            },
            function (callback) {
                // Asserts dialog box disappears
                client.getAlertText(function (result) {
                    // Assumes there is no content
                    if (result.value !== 'Are you sure to delete file sample.txt?') {
                        callback();
                    }
                    else { // Terminate test
                        client.assert.fail(result.value, "", "Alert should not be visible");
                    }
                });
            },
            function (callback) {
                // Clicks delete button, again
                fileManagement.click('@delBtn');

                client.pause(1000);

                // Assumes alert box appears. Clicks OK.
                client.acceptAlert(function () {
                    callback();
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-008
     *  Expects:
     *      - "sample.txt" will be deleted from the file list
     *      - If there will be no files left, "No Records Found" label should be displayed in the file list
     *      - Using the Windows explorer or terminal, go to Hark Designer's userdata/networks directory and confirm that "sample.txt" is already deleted.
     */
    "1-008": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-008
                client.pause(1000,function () {
                    // Gets the latest file in the directory to confirm deleted
                    var file = fs.readdirSync(defaultDir + '/');

                    if (file.length <= 0) {
                        // Assert file list if there is no content after deleting the file
                        fileManagement.expect.element('@fileList').text.to.contain('No Records Found');

                        callback();
                    }
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-009
     *  Expects:
     *      - In the file list, all 5 files should be displayed alphabetically according to type
     *      - Pagination should display first page only
     *      - 1, first, previous, next, and last buttons are disabled
     */
    "1-009": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-009
                // Create Fixtures
                var files = generateFileName(fileListCount);

                files = _.map(files, function (file) {
                    return file + ".txt";
                });

                for (var i = 0; i < files.length; i++) {
                    var path = defaultDir + "/" + files[i];
                    fs.outputFileSync(path, "");
                }
                sampleFiles = sampleFiles.concat(files);

                sortByType();

                // Clicks reload button
                fileManagement.clickReload();

                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Assert files are listed
                    var files = getPagedData(1);
                    for (var i = 0; i < fileListCount; i++) {
                        fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                    }

                    // Asserts page button to display '1'
                    fileManagement.expect.element('@pages').text.to.contain('1');

                    // Assert navigation buttons to not clickable
                    fileManagement.expect.element('@pageOne').to.have.attribute('class')
                        .which.contains('active');
                    fileManagement.expect.element('@firstPage').to.have.attribute('class')
                        .which.contains('disabled');
                    fileManagement.expect.element('@prevPage').to.have.attribute('class')
                        .which.contains('disabled');
                    fileManagement.expect.element('@nextPage').to.have.attribute('class')
                        .which.contains('disabled');
                    fileManagement.expect.element('@lastPage').to.have.attribute('class')
                        .which.contains('disabled');
                });

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-010
     *  Expects:
     *      - In the file list, first 5 files should be displayed alphabetically according to type in page 1
     *      - Pagination should display pages 1 and 2
     *      - 1, first, and previous buttons are disabled and next and last buttons are enabled
     */
    "1-010": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-010
                client.pause(1000, function () {
                    // Adds another fixture
                    var files = generateFileName(fileListCount);

                    files = _.map(files, function (file) {
                        return file + ".txt";
                    });

                    for (var i = 0; i < files.length; i++) {
                        var path = defaultDir + "/" + files[i];
                        fs.outputFileSync(path, "");
                    }

                    sampleFiles = sampleFiles.concat(files);

                    sortByType();

                    // Clicks reload button
                    fileManagement.clickReload();
                });

                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Assert files are listed
                    var files = getPagedData(1);
                    for (var i = 0; i < fileListCount; i++) {
                        fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                    }

                    // Asserts page buttons are displayed
                    fileManagement.expect.element('@pages').text.to.contain('1');
                    fileManagement.expect.element('@pages').text.to.contain('2');

                    // Asserts these navigation buttons are not clickable
                    fileManagement.expect.element('@pageOne').to.have.attribute('class')
                        .which.contains('active');
                    fileManagement.expect.element('@firstPage').to.have.attribute('class')
                        .which.contains('disabled');
                    fileManagement.expect.element('@prevPage').to.have.attribute('class')
                        .which.contains('disabled');

                    // Asserts these navigation buttons are clickable
                    fileManagement.expect.element('@pageTwo2').to.have.attribute('class')
                        .which.not.contains('active');
                    fileManagement.expect.element('@nextPage2').to.have.attribute('class')
                        .which.not.contains('disabled');
                    fileManagement.expect.element('@lastPage2').to.have.attribute('class')
                        .which.not.contains('disabled');
                });

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-011
     *  Expects:
     *      - In the file list, second 5 files should be displayed alphabetically according to type in page 2
     *      - Pagination should display pages 1 and 2
     *      - 1, first, and previous buttons are enabled and 2, next, and last buttons are disabled
     */
    "1-011": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-011
                fileManagement.click('@pageTwo2anchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert second paged files if listed
                var files = getPagedData(2);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons displayed
                fileManagement.expect.element('@pages').text.to.contain('1');
                fileManagement.expect.element('@pages').text.to.contain('2');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageTwo2').to.have.attribute('class')
                    .which.contains('active');
                fileManagement.expect.element('@nextPage2').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@lastPage2').to.have.attribute('class')
                    .which.contains('disabled');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-012
     *  Expects:
     *      - In the file list, second 5 files should be displayed alphabetically according to type in page 2
     *      - Pagination should display pages 1 and 2
     *      - 1, first, and previous buttons are enabled and 2, next, and last buttons are disabled
     */
    "1-012": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-012
                fileManagement.click('@nextPage2anchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert second paged files if listed
                var files = getPagedData(2);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons displayed
                fileManagement.expect.element('@pages').text.to.contain('1');
                fileManagement.expect.element('@pages').text.to.contain('2');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageTwo2').to.have.attribute('class')
                    .which.contains('active');
                fileManagement.expect.element('@nextPage2').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@lastPage2').to.have.attribute('class')
                    .which.contains('disabled');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-013
     *  Expects:
     *      - In the file list, second 5 files should be displayed alphabetically according to type in page 2
     *      - Pagination should display pages 1 and 2
     *      - 1, first, and previous buttons are enabled and 2, next, and last buttons are disabled
     */
    "1-013": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-013
                fileManagement.click('@lastPage2anchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert second paged files if listed
                var files = getPagedData(2);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons displayed
                fileManagement.expect.element('@pages').text.to.contain('1');
                fileManagement.expect.element('@pages').text.to.contain('2');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageTwo2').to.have.attribute('class')
                    .which.contains('active');
                fileManagement.expect.element('@nextPage2').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@lastPage2').to.have.attribute('class')
                    .which.contains('disabled');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-014
     *  Expects:
     *      - In the file list, first 5 files should be displayed alphabetically according to type in page 1
     *      - Pagination should display pages 1 and 2
     *      - 1, first, and previous buttons are disabled and 2, next, and last buttons are enabled
     */
    "1-014": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-014
                fileManagement.click('@pageOneAnchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert first paged files if listed
                var files = getPagedData(1);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons displayed
                fileManagement.expect.element('@pages').text.to.contain('1');
                fileManagement.expect.element('@pages').text.to.contain('2');

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.contains('disabled');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@pageTwo2').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@nextPage2').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@lastPage2').to.have.attribute('class')
                    .which.not.contains('disabled');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-015
     *  Expects:
     *      - In the file list, first 5 files should be displayed alphabetically according to type in page 1
     *      - Pagination should display pages 1 and 2
     *      - 1, first, and previous buttons are disabled and 2, next, and last buttons are enabled
     */
    "1-015": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-015
                fileManagement.click('@prevPageAnchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert first paged files if listed
                var files = getPagedData(1);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons displayed
                fileManagement.expect.element('@pages').text.to.contain('1');
                fileManagement.expect.element('@pages').text.to.contain('2');

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.contains('disabled');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@pageTwo2').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@nextPage2').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@lastPage2').to.have.attribute('class')
                    .which.not.contains('disabled');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-016
     *      - In the file list, first 5 files should be displayed alphabetically according to type in page 1
     *      - Pagination should display pages 1 and 2
     *      - 1, first, and previous buttons are disabled and next and last buttons are enabled
     */
    "1-016": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-016
                fileManagement.click('@firstPageAnchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert first paged files if listed
                var files = getPagedData(1);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons displayed
                fileManagement.expect.element('@pages').text.to.contain('1');
                fileManagement.expect.element('@pages').text.to.contain('2');

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.contains('disabled');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@pageTwo2').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@nextPage2').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@lastPage2').to.have.attribute('class')
                    .which.not.contains('disabled');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-017
     *  Expects:
     *      - In the file list, "sample.n" must be included
     *      - Pagination should display pages 1, 2, and 3
     *      - 1, first, and previous buttons are disabled and 2, 3, next, and last buttons are enabled
     */
    "1-017": function (client) {
        var fileManagement = client.page.fileManagement();
        var filePath = fixtureDir + '/sample.n';

        async.series([
            function (callback) { // Test: 1-017
                // Copy file
                fs.copySync(filePath, defaultDir + '/sample.n');

                sampleFiles = sampleFiles.concat('sample.n');

                sortByType();

                // Clicks reload button
                fileManagement.clickReload();

                client.pause(1000, function () {
                    callback();
                });
            },
            function (callback) {
                // Assert file created present in file list
                fileManagement.expect.element('@fileList').text.to.contain('sample.n');

                // Asserts page buttons displayed
                fileManagement.expect.element('@pages').text.to.contain('1');
                fileManagement.expect.element('@pages').text.to.contain('2');
                fileManagement.expect.element('@pages').text.to.contain('3');

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@pageTwo3').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@pageThree3').to.have.attribute('class')
                    .which.not.contains('active');
                fileManagement.expect.element('@nextPage3').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@lastPage3').to.have.attribute('class')
                    .which.not.contains('disabled');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-018
     *  Expects:
     *      - In the file list, "sample.n" will be the only result displayed.
     */
    "1-018": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Clear value for filter
                fileManagement.clearValue('@filter');
                // Select filter
                fileManagement.moveToElement('@filter', 10, 10);

                callback();
            },
            function (callback) {
                // Enter 'sample.n' key
                client.keys('sample.n',function () {
                    this.pause(1000);
                    callback();
                });
            },
            function (callback) {
                // Assert file list if file with 'sample.n' is displayed
                fileManagement.expect.element('@fileList').text.to.contain('sample.n');

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-019
     *  Expects:
     *      - "sample.n" network should be loaded in Hark Designer.
     */
    "1-019": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Click open network button
                fileManagement.click('@openNfileBtn', function () {
                    this.pause(1000);
                    callback();
                });
            },
            function (callback) {
                // Accepts alert box
                client.acceptAlert(function () {
                    this.pause(1000);
                    callback();
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    // Assert network file is loaded
                    fileManagement.expect.element('@consoleStatus').text.to.contain('Loading finished.').after(1000);
                    fileManagement.expect.element('@fileName').text.to.contain('sample.n').after(1000);

                    callback();
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-020
     *  Expects:
     *      - In the file list, "invalid.n" must be included
     *      - Pagination should display pages 1, 2, and 3
     *      - 1, first, and previous buttons are disabled and 2, 3, next, and last buttons are enabled
     */
    "1-020": function (client) {
        var fileManagement = client.page.fileManagement();
        var filePath = fixtureDir + '/invalid.n';

        async.series([
            function (callback) { // Test: 1-020
                // Copy file
                fs.copySync(filePath, defaultDir + '/invalid.n');

                sampleFiles = sampleFiles.concat('invalid.n');

                sortByType();

                // Open file manager
                fileManagement.open();

                callback();
            },
            function (callback) {
                // Clicks reload button
                fileManagement.clickReload();

                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Assert first page files are shown
                    var files = getPagedData(1);
                    for (var i = 0; i < fileListCount; i++) {
                        fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                    }

                    // Asserts page buttons are displayed from 1 to 10
                    for (var i = 1; i <= 3; i++) {
                        fileManagement.expect.element('@pages').text.to.contain(i);
                    }

                    // Asserts these navigation buttons are not clickable
                    fileManagement.expect.element('@firstPage').to.have.attribute('class')
                        .which.contains('disabled');
                    fileManagement.expect.element('@prevPage').to.have.attribute('class')
                        .which.contains('disabled');
                    fileManagement.expect.element('@pageOne').to.have.attribute('class')
                        .which.contains('active');

                    // Asserts these navigation buttons are clickable
                    for (var i = 4; i <= 7; i++) {
                        if (i < 13) {
                            client
                                .useXpath()
                                .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                                .which.not.contains('active');
                        }
                        else {
                            client
                                .useXpath()
                                .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                                .which.not.contains('disabled');
                        }
                    }

                    callback();
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-021
     *  Expects:
     *      - In the file list, "invalid.n" will be the only result displayed.
     */
    "1-021": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-021
                // Select file path
                fileManagement.moveToElement('@filter', 10, 10);

                // Clear file path value
                fileManagement.clearValue('@filter');

                // Set file path value
                fileManagement.setValue('@filter', 'invalid.n');

                // Hit enter key
                client.keys(client.Keys.ENTER, function () {
                    client.pause(1000, function () {
                        // Assert files if present in file list
                        fileManagement.expect.element('@fileList').text.to.contain('invalid.n').after(1000);
                    });
                });

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**

     *  Test: 1-022
     *  Expects:
     *      - In the console, error should be displayed that it cannot load the n file due to parsing error.
     */
    "1-022": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-022
                // Select file path
                fileManagement.moveToElement('@openNfileBtn', 10, 10);

                // Open invalid network
                fileManagement.click('@openNfileBtn', function () {
                    client.pause(1000, function () {
                        callback();
                    });
                });
            },
            function (callback) {
                // Accepts alert
                client.acceptAlert(function () {
                    callback();
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    // Assert to contain if error message is present
                    fileManagement.expect.element('@consoleStatus').text.to.not.equals('');

                    callback();
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-023
     *  Expects:
     *      - In the file list, first 5 files should be displayed alphabetically according to type in page 1
     *      - Pagination should display pages 1 to 10 initially
     *      - 1, first, and previous buttons are disabled and next, last, and page 2 to 10 buttons are enabled
     */
    "1-023": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-023
                // Create Fixtures
                var files = generateFileName(48);

                files = _.map(files, function (file) {
                    return file + ".txt";
                });

                for (var i = 0; i < files.length; i++) {
                    var path = defaultDir + "/" + files[i];
                    fs.outputFileSync(path, "");
                }

                sampleFiles = sampleFiles.concat(files);

                sortByType();

                client.pause(1000, function () {
                    // Open file manager
                    fileManagement.open();

                    callback();
                });
            },
            function (callback) {
                // Assert first page files are shown
                var files = getPagedData(1);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons are displayed from 1 to 10
                for (var i = 1; i <= 10; i++) {
                    fileManagement.expect.element('@pages').text.to.contain(i);
                }

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');

                // Asserts these navigation buttons are clickable
                for (var i = 4; i <= 14; i++) {
                    if (i < 13) {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                                .which.not.contains('active');
                    }
                    else {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                                .which.not.contains('disabled');
                    }
                }

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-024
     *  Expects:
     *      - In the file list, second 5 files should be displayed alphabetically according to type in page 2
     *      - Pagination should display pages 2 to 11
     *      - 2 is disabled and pages 3 to 11, first, previous, next, and last buttons are  enabled
     */
    "1-024": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-024
                fileManagement.click('@nextPage4anchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert second page files are shown
                var files = getPagedData(2);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons are displayed from 2 to 11
                for (var i = 2; i <= 11; i++) {
                    fileManagement.expect.element('@pages').text.to.contain(i);
                }

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');

                for (var i = 4; i <= 14; i++) {
                    if (i < 13) {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('active');
                    }
                    else {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('disabled');
                    }
                }

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-025
     *  Expects:
     *      - In the file list, third 5 files should be displayed alphabetically according to type  in page 3
     *      - Pagination should display pages 3 to 12
     *      - 3 is disabled and pages 4 to 12, first, previous, next, and last buttons are  enabled
     */
    "1-025": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-025
                fileManagement.click('@nextPage4anchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert second page files are shown
                var files = getPagedData(3);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons are displayed from 3 to 12
                for (var i = 3; i <= 12; i++) {
                    fileManagement.expect.element('@pages').text.to.contain(i);
                }

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');

                for (var i = 4; i <= 14; i++) {
                    if (i < 13) {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('active');
                    }
                    else {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('disabled');
                    }
                }

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-026
     *  Expects:
     *      - In the file list, fourth 5 files should be displayed alphabetically according to type in page 4
     *      - Pagination should display pages 3 to 12
     *      - 4 is disabled and pages 3 and 5 to 12, first, previous, next, and last buttons are  enabled
     */
    "1-026": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-026
                fileManagement.click('@nextPage4anchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert second page files are shown
                var files = getPagedData(4);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons are displayed from 3 to 12
                for (var i = 3; i <= 12; i++) {
                    fileManagement.expect.element('@pages').text.to.contain(i);
                }

                // Assert if not clickable
                fileManagement.expect.element('@pageFour4').to.have.attribute('class')
                    .which.contains('active');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.not.contains('active');

                for (var i = 5; i <= 14; i++) {
                    if (i < 13) {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('active');
                    }
                    else {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('disabled');
                    }
                }

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-027
     *  Expects:
     *      - In the file list, second 5 files should be displayed alphabetically according to type in page 2
     *      - Pagination should display pages 2 to 11
     *      - 2 is disabled and pages 3 to 11, first, previous, next, and last buttons are  enabled
     */
    "1-027": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-027
                fileManagement.click('@prevPageAnchor', function () {
                    fileManagement.click('@prevPageAnchor', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                // Assert second page files are shown
                var files = getPagedData(2);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons are displayed from 2 to 11
                for (var i = 2; i <= 11; i++) {
                    fileManagement.expect.element('@pages').text.to.contain(i);
                }

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');

                for (var i = 4; i <= 14; i++) {
                    if (i < 13) {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('active');
                    }
                    else {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('disabled');
                    }
                }

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-028
     *  Expects:
     *      - In the file list, last 5 files should be displayed alphabetically according to type in page 12
     *      - Pagination should display pages 3 and 12
     *      - 3 to 11, first, and previous buttons are enabled and 12, next, and last buttons are disabled
     */
    "1-028": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-028
                fileManagement.click('@lastPage4anchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert files are shown
                var files = getPagedData(12);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons are displayed from 2 to 11
                for (var i = 3; i <= 12; i++) {
                    fileManagement.expect.element('@pages').text.to.contain(i);
                }

                // Asserts these navigation buttons are clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.not.contains('disabled');
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.not.contains('active');

                for (var i = 4; i <= 14; i++) {
                    if (i < 13) {
                        if (i === 12) {
                            client
                                .useXpath()
                                .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                                .which.contains('active');
                        }
                        else {
                            client
                                .useXpath()
                                .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                                .which.not.contains('active');
                        }
                    }
                    else {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.contains('disabled');
                    }
                }

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 1-029
     *  Expects:
     *      - In the file list, first 5 files should be displayed alphabetically according to type in page 1
     *      - Pagination should display pages 1 to 10 initially
     *      - 1, first, and previous buttons are disabled and next, last, and page 2 to 10 buttons are enabled
     */
    "1-029": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) { // Test: 1-029
                fileManagement.click('@firstPageAnchor', function () {
                    callback();
                });
            },
            function (callback) {
                // Assert files are shown
                var files = getPagedData(1);
                for (var i = 0; i < fileListCount; i++) {
                    fileManagement.expect.element('@fileList').text.to.contain(files[i]);
                }

                // Asserts page buttons are displayed from 2 to 11
                for (var i = 1; i <= 10; i++) {
                    fileManagement.expect.element('@pages').text.to.contain(i);
                }

                // Asserts these navigation buttons are not clickable
                fileManagement.expect.element('@firstPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@prevPage').to.have.attribute('class')
                    .which.contains('disabled');
                fileManagement.expect.element('@pageOne').to.have.attribute('class')
                    .which.contains('active');

                // Asserts these navigation buttons are clickable
                for (var i = 4; i <= 14; i++) {
                    if (i < 13) {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('active');
                    }
                    else {
                        client
                            .useXpath()
                            .expect.element('//*[@id="paginator"]/div/ul/li['+ i +']').to.have.attribute('class')
                            .which.not.contains('disabled');
                    }
                }

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     * Test: 1-030
     *  Expects:
     *      - Search input should be cleared
     *      - File list should display page 1
     */
    "1-030": function (client) {
        var fileManagement = client.page.fileManagement();

        fileManagement.close();
        fileManagement.open();

        fileManagement.expect.element('@filter').value.to.equal('');

        // / Assert first page files are shown
        var files = getPagedData(1);
        for (var i = 0; i < fileListCount; i++) {
            fileManagement.expect.element('@fileList').text.to.contain(files[i]);
        }
    }
};