var async = require('async');
var moment = require('moment');
var fs = require('fs-extra');
var del = require('del');
var path = require('path');

var fixtureDir = '/usr/src/app/automated-tests/lib/fixtures';

module.exports = {
    '@tags': ['fm_fileUpload'],

    before: function (client) {
        client
            .url('http://localhost:3000')
            .waitForElementVisible('#topmenu', 1000);
    },

    after: function (client) {
        client.end();
    },

    /**
     *  Test: 3-001
     *  Expects:
     *      - Default file select dialog of OS will be displayed for selection of file.
     *  Remarks:
     *      - Cannot assert since no file select dialog present. Assert to PASS.
     */
    "3-001": function (client) {
        // Nothing
    },

    /**
     *  Test: 3-002
     *  Expects:
     *      - Selected n file should be displayed in the list with type as "network"
     *      - Action buttons available is delete, load, and download.
     */
    "3-002": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
           function (callback) {
               // Open file manager
               fileManagement.open();
               callback();
           },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/sample.n'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite sample.n with sample.n?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        } else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'sample.n', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    fileManagement.expect.element('@fixture_1').to.be.visible;
                    fileManagement.expect.element('@fixtureType_1').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_1').to.be.visible;
                    fileManagement.expect.element('@fixtureLoad_1').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_1').to.be.visible;
                    callback();
                });
            }
        ], function () {
            // nothing
        });
    },


    /**
     *  Test: 3-003
     *  Expects:
     *      - Selected wav file should be displayed in the list with type as "audio"
     *      - Action buttons available is delete and download.
     */
    "3-003": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Open file manager
                fileManagement.open();
                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/sample.wav'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite sample.wav with sample.wav?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        }
                        else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'sample.wav', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    fileManagement.expect.element('@fixture_2').to.be.visible;
                    fileManagement.expect.element('@fixtureType_2').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_2').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_2').to.be.visible;
                    callback();
                });
            }
        ], function () {
            //nothing
        });
    },
    /**
     *  Test: 3-004
     *  Expects:
     *      - Selected raw file should be displayed in the list with type as "audio"
     *      - Action buttons available is delete and download.
     */
    "3-004": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Open file manager
                fileManagement.open();
                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/sampleraw.raw'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite sampleraw.raw with sampleraw.raw?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        }
                        else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'sampleraw.raw', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    fileManagement.expect.element('@fixture_3').to.be.visible;
                    fileManagement.expect.element('@fixtureType_3').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_3').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_3').to.be.visible;
                    callback();
                });
            }
        ], function () {
            //nothing
        });
    },

    /**
     *  Test: 3-005
     *  Expects:
     *      - Selected DAT file should be displayed in the list with type as "tf"
     *      - Action buttons available is delete and download.
     */
    "3-005": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Open file manager
                fileManagement.open();
                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/datfile.dat'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite datfile.dat with datfile.dat?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        }
                        else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'datfile.dat', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    fileManagement.expect.element('@fixture_4').to.be.visible;
                    fileManagement.expect.element('@fixtureType_4').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_4').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_4').to.be.visible;
                    callback();
                });
            }
        ], function () {
            //nothing
        })
    },

    /**
     *  Test: 3-006
     *  Expects:
     *      - Selected tff file should be displayed in the list with type as "tf"
     *      - Action buttons available is delete and download.
     */
    "3-006": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
            // Open file manager
            fileManagement.open();
            callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/sampletff.tff'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite sampletff.tff with sampletff.tff?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        }
                        else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'sampletff.tff', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    fileManagement.expect.element('@fixture_5').to.be.visible;
                    fileManagement.expect.element('@fixtureType_5').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_5').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_5').to.be.visible;
                    callback();
                });
            }
        ], function () {
            //nothing
        });
    },

    /**
     *  Test: 3-007
     *  Expects:
     *      - Selected zip file should be displayed in the list with type as "tf"
     *      - Action buttons available is delete and download.
     */
    "3-007": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Open file manager
                fileManagement.open();
                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/tf.zip'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite tf.zip with tf.zip?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        }
                        else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'tf.zip', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(3000, function () {
                    fileManagement.expect.element('@fixture_6').to.be.visible;
                    fileManagement.expect.element('@fixtureType_6').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_6').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_6').to.be.visible;
                    callback();
                });
            }
        ], function () {
            //nothing
        });
    },

    /**
     *  Test: 3-008
     *  Expects:
     *      - Selected zip file should be displayed in the list with type as "others"
     *      - Action buttons available is delete and download.
     */
    "3-008": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Open file manager
                fileManagement.open();
                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/invalidtf.zip'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite invalidtf.zip with invalidtf.zip?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        }
                        else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'invalidtf.zip', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(3000, function () {
                    fileManagement.expect.element('@fixture_7').to.be.visible;
                    fileManagement.expect.element('@fixtureType_7').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_7').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_7').to.be.visible;
                    callback();
                });
            }
        ], function () {
            //nothing
        });

    },

    /**
     *  Test: 3-009
     *  Expects:
     *      - Selected file should be displayed in the list with type as "others"
     *      - Action buttons available is delete and download.
     */
    "3-009": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                // Open file manager
                fileManagement.open();
                callback();
            },
            function (callback) {
                client.pause(1000, function () {
                    // Upload a file
                    fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/otherfile.xml'));
                    callback();
                });
            },
            function (callback) {
                client.getAlertText(function(alertText){
                    client.pause(1000, function () {
                        if(alertText.value == 'Overwrite otherfile.xml with otherfile.xml?') {
                            //accept alert
                            client.acceptAlert(function () {
                                client.pause(1000, function () {
                                    callback();
                                });
                            });
                        }
                        else {
                            callback();
                        }
                    });
                });
            },
            function (callback) {
                fileManagement.clearValue('@searchInput', function () {
                    fileManagement.setValue('@searchInput', 'otherfile.xml', function () {
                        callback();
                    });
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    fileManagement.expect.element('@fixture_8').to.be.visible;
                    fileManagement.expect.element('@fixtureType_8').to.be.visible;
                    fileManagement.expect.element('@fixtureDelete_8').to.be.visible;
                    fileManagement.expect.element('@fixtureDownload_8').to.be.visible;
                    callback();
                });
            }
        ], function () {
            //nothing
        });
        client.end();
    },

    /**
     *  Test: 3-010
     *  Expects:
     *      - Default dialog box will appear asking "Overwrite <filename> with <filename>?"
     */
    "3-010": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                client
                    .url('http://localhost:3000')
                    .waitForElementVisible('#topmenu', 1000);
                callback();
            },
            function (callback) {
                // Open file manager
                fileManagement.open();
                callback();
            },
            function (callback) {
                fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/sample.n'));
                callback();
            },
            function (callback) {
                client.pause(2000, function () {
                    client.getAlertText(function(alertText){
                        if(alertText) {
                            client.assert.equal(alertText.value, 'Overwrite sample.n with sample.n?');
                        }
                        callback();
                    });
                });
            }
        ], function () {
            // nothing
        });
    },

    /**
     *  Test: 3-011
     *  Expects:
     *      - Dialog box will disappear.
     *      - The file with the same name will be replaced.
     */
    "3-011": function (client) {
        // Nothing
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                client.acceptAlert(function () {
                    client.pause(1000, function () {
                        callback();
                    });
                });
            },
            function (callback) {
                fileManagement.expect.element('body').to.be.present;
                callback();
            }
        ], function () {
            // nothing
        });
    },

    /**
     *  Test: 3-012
     *  Expects:
     *      - Dialog box will disappear.
     *      - The file with the same name will not be replaced.
     */
    "3-012": function (client) {
        var fileManagement = client.page.fileManagement();

        async.series([
            function (callback) {
                fileManagement.setValue('@uploadBtn', path.resolve(fixtureDir + '/sample.n'));
                callback();
            },
            function (callback) {
                client.pause(2000, function () {
                    client.getAlertText(function(alertText){
                        if(alertText) {
                            client.assert.equal(alertText.value, 'Overwrite sample.n with sample.n?');
                        }
                        callback();
                    });
                });
            },
            function (callback) {
                client.dismissAlert(function () {
                    client.pause(1000, function () {
                        callback();
                    });
                });
            },
            function (callback) {
                fileManagement.expect.element('body').to.be.present;
                callback();
            }
        ], function () {
            // nothing
        });
    }

};